package cz.novoj.hugeImport;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.support.AbstractInterruptibleBatchPreparedStatementSetter;
import org.springframework.transaction.annotation.Transactional;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.text.SimpleDateFormat;

/**
 * Description
 *
 * @author Jan Novotn, FG Forrest a.s. (c) 2007
 * @version $Id: $
 */
public abstract class AbstractImportLogic {
	private static final Log log = LogFactory.getLog(AbstractImportLogic.class);
	private JdbcTemplate jdbcTemplate;
	private int iterationCount = 100000;

	public abstract String getImplementation();

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}

	public void setIterationCount(int iterationCount) {
		this.iterationCount = iterationCount;
	}

	public int getIterationCount() {
		return iterationCount;
	}

	public void naiveImport() {
		internalSingleRowLogic(1, iterationCount);
	}

	public void multiRowInsert(int partNumber, int partSize) {
		internalMultiRowLogic((partNumber - 1) * partSize + 1, partSize);
	}

	@Transactional
	public void multiRowInsertUnderTransaction(int partNumber, int partSize) {
		internalMultiRowLogic((partNumber - 1) * partSize + 1, partSize);
	}

	@Transactional
	public void importUnderTransaction() {
		internalSingleRowLogic(1, iterationCount);
	}

	@Transactional
	public void importUnderTransactionPerPartes(int partNumber, int partSize) {
		internalSingleRowLogic((partNumber - 1) * partSize + 1, partSize);
	}

	public void importLoadDataFromFile() throws Exception {
		StringBuilder table1Contents = new StringBuilder();
		String date = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(System.currentTimeMillis()));
		for (int i = 0; i < iterationCount; i++) {
			table1Contents.append("\"").append(i + 1).append("\",");
			table1Contents.append("\"").append("novotnaci").append(i).append("@gmail.com").append("\",");
			table1Contents.append("\"").append(date).append("\"\n");
		}
		StringBuilder table2Contents = new StringBuilder();
		for (int i = 0; i < iterationCount; i++) {
			table2Contents.append("\"").append(i + 1).append("\",");
			table2Contents.append("\"").append(i + 1).append("\",");
			table2Contents.append("\"").append("Jan Novotn").append("\"\n");
		}
		//store files
		String recipientFile = System.getProperty("java.io.tmpdir") + "T_RECIPIENT_" + getImplementation() + ".sql";
		storeFile(table1Contents.toString(), recipientFile);
		String recipientPropsFile = System.getProperty("java.io.tmpdir") + "T_RECIPIENT_PROPS_" + getImplementation() + ".sql";
		storeFile(table2Contents.toString(), recipientPropsFile);
		//execute load data
		String sqlRecipient = "LOAD DATA INFILE '" + recipientFile.replaceAll("\\\\", "/") + "' INTO TABLE T_RECIPIENT_" + getImplementation() + "\n" +
				"FIELDS TERMINATED BY ',' ENCLOSED BY '\"' ESCAPED BY '\\\\'\n" +
				"LINES TERMINATED BY '\\n'";
		log.info("SQL: " + sqlRecipient);
		jdbcTemplate.execute(sqlRecipient);
		String sqlRecipientProps = "LOAD DATA INFILE '" + recipientPropsFile.replaceAll("\\\\", "/") + "' INTO TABLE T_RECIPIENT_PROPS_" + getImplementation() + "\n" +
				"FIELDS TERMINATED BY ',' ENCLOSED BY '\"' ESCAPED BY '\\\\'\n" +
				"LINES TERMINATED BY '\\n'";
		log.info("SQL: " + sqlRecipientProps);
		jdbcTemplate.execute(sqlRecipientProps);
	}

	private void storeFile(String contents, String filename) throws IOException {
		OutputStream os = null;
		try {
			os = new BufferedOutputStream(new FileOutputStream(filename));
			IOUtils.write(contents, os, "UTF-8");
		} finally {
			IOUtils.closeQuietly(os);
		}
	}

	private void internalSingleRowLogic(final int fromRecord, final int maxRecords) {
		jdbcTemplate.batchUpdate(
				"insert into T_RECIPIENT_" + getImplementation() + " (id, email, created) values (?, ?, ?)",
				new AbstractInterruptibleBatchPreparedStatementSetter() {
					private int iteration = fromRecord - 1;

					@Override
					protected boolean setValuesIfAvailable(PreparedStatement ps, int i) throws SQLException {
						iteration++;
						if (iteration < fromRecord + maxRecords) {
							ps.setInt(1, iteration);
							ps.setString(2, "novotnaci" + iteration + "@gmail.com");
							ps.setDate(3, new Date(System.currentTimeMillis()));
							return true;
						} else {
							return false;
						}
					}
				}
		);
		jdbcTemplate.batchUpdate(
				"insert into T_RECIPIENT_PROPS_" + getImplementation() + " (idProp, idRcpt, name) values (?, ?, ?)",
				new AbstractInterruptibleBatchPreparedStatementSetter() {
					private int iteration = fromRecord - 1;

					@Override
					protected boolean setValuesIfAvailable(PreparedStatement ps, int i) throws SQLException {
						iteration++;
						if (iteration < fromRecord + maxRecords) {
							ps.setInt(1, iteration);
							ps.setInt(2, iteration);
							ps.setString(3, "Jan Novotn");
							return true;
						} else {
							return false;
						}
					}
				}
		);
	}

	private void internalMultiRowLogic(int fromRecord, int maxRecords) {
		StringBuilder mainInsert = new StringBuilder("insert into T_RECIPIENT_" + getImplementation() +" (id, email, created) values ");
		Object[] mainParameters = new Object[maxRecords * 3];
		StringBuilder relatedInsert = new StringBuilder("insert into T_RECIPIENT_PROPS_" + getImplementation() + " (idProp, idRcpt, name) values ");
		Object[] relatedParameters = new Object[maxRecords * 3];

		for(int i = fromRecord; i < fromRecord + maxRecords; i ++) {
			mainInsert.append("(?,?,?)");
			mainParameters[(i - fromRecord) * 3] = i;
			mainParameters[(i - fromRecord) * 3 + 1] = "novotnaci" + i + "@gmail.com";
			mainParameters[(i - fromRecord) * 3 + 2] = new Date(System.currentTimeMillis());
			relatedInsert.append("(?,?,?)");
			relatedParameters[(i - fromRecord) * 3] = i;
			relatedParameters[(i - fromRecord) * 3 + 1] = i;
			relatedParameters[(i - fromRecord) * 3 + 2] = "Jan Novotn";
			if (i + 1 < fromRecord + maxRecords) {
				mainInsert.append(", ");
				relatedInsert.append(", ");
			}
		}

		log.info("SQL: " + mainInsert);
		jdbcTemplate.update(mainInsert.toString(), mainParameters);
		log.info("SQL: " + relatedInsert);
		jdbcTemplate.update(relatedInsert.toString(), relatedParameters);
	}
}
